/**
 * Copyright 2007-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.impl;

import org.ow2.util.pool.api.PoolException;

/**
 * Allows to call specific methods for each implementation of the pool for each lifecycle on an object.
 * @param <InstanceType> the type of the object that are managed by the pool
 *        (could be an EJB, JDBC connection, etc.)
 * @param <Clue> a clue which is given to match an instance in the pool
 * @author Florent Benoitt
 */
public interface PoolFactory<InstanceType, Clue> {


    /**
     * Creates an instance with the given hint.
     * @param clue a clue given by the Pool. Could be null.
     * @throws PoolException if instance cannot be created.
     * @return the created instance.
     */
    InstanceType create(Clue clue) throws PoolException;

    /**
     * Checks if the given object with the given clue is matching.
     * @param instance given object against which the check should be done.
     * @param clue the object used as clue to check the matching.
     * @return true if it is matching, else false.
     */
    boolean isMatching(InstanceType instance, Clue clue);

    /**
     * Callback called when object is gonna be removed.
     * @param instance that is being removed from the pool.
     */
    void remove(InstanceType instance);

    /**
     * Sets the given clue on the given instance.
     * If no clue is used, this method shouldn't do anything
     * @param instance the instance to be updated with the clue
     * @param clue the given clue
     * @throws PoolException if clue can't be set
     */
    //void setClue(InstanceType instance, Clue clue) throws PoolException;


    /**
     * Validate an instance by giving some statistics.
     * @param instance the instance to validate
     * @param stats some statistics to help in the validating process.
     * @return true if the element is valid, else false.
     */
    boolean validate(InstanceType instance, PoolEntryStatistics stats);

    
}
